/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka.model;

import com.fasterxml.jackson.annotation.JsonProperty;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;

/**
 * POJO representing a job for the datafetcher created by the service
 */
public class DataFetcherJob {

    private final String msgId;

    private final JobType type;

    private final String jobId;

    private final String cpr;

    public DataFetcherJob(@JsonProperty("msgId") String msgId,
                          @JsonProperty("type") JobType type,
                          @JsonProperty("jobId") String jobId,
                          @JsonProperty("cpr") String cpr) {
        this.msgId = msgId;
        this.type = type;
        this.jobId = jobId;
        this.cpr = cpr;
    }

    public String getMsgId() {
        return msgId;
    }

    public JobType getType() {
        return type;
    }

    public String getJobId() {
        return jobId;
    }

    public String getCpr() {
        return cpr;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;

        if (o == null || getClass() != o.getClass()) return false;

        DataFetcherJob that = (DataFetcherJob) o;

        return new EqualsBuilder().append(msgId, that.msgId).append(type, that.type).append(jobId, that.jobId).append(cpr, that.cpr).isEquals();
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder(17, 37).append(msgId).append(type).append(jobId).append(cpr).toHashCode();
    }

    @Override
    public String toString() {
        return "DataFetcherJob{" +
                "msgId='" + msgId + '\'' +
                ", type=" + type +
                ", jobId='" + jobId + '\'' +
                ", cpr='<redacted>" + '\'' +
                '}';
    }
}
