/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka;

import dk.sds.nsp.kafka.clients.producer.NSPKafkaProducer;
import dk.sundhedsdatastyrelsen.cbs.kafka.model.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.kafka.model.JobType;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import dk.sundhedsdatastyrelsen.cbs.test.TestUtils;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.serialization.Serializer;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;

import java.util.Properties;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@CbsTest
@SuppressWarnings("java:S5979") /*SonarLint cannot see that @CbsTest has @ExtendWith(MockitoExtension.class)*/
class CbsKafkaProducerTest {
    @Mock
    private NSPKafkaProducer<String, DataFetcherJob> mockNspProducer;
    @Mock
    private CbsKafkaFactory<String, DataFetcherJob> mockFactory;
    @Captor
    private ArgumentCaptor<ProducerRecord<String, DataFetcherJob>> producerRecordArgumentCaptor;

    @BeforeEach
    void setUp() {
        when(mockFactory.getNSPKafkaProducer(any(Properties.class), any(Serializer.class), any(Serializer.class))).thenReturn(mockNspProducer);
    }

    @Test
    void test_send_job() {
        String topic = "topic";
        CbsKafkaProducer cbsKafkaProducer = new CbsKafkaProducer(topic, new Properties(), mockFactory);

        String cpr = TestUtils.randomCprNumber();
        JobType jobType = JobType.FETCH_VACCINATION_STATUS;
        String jobId = UUID.randomUUID().toString();
        String msgId = UUID.randomUUID().toString();

        cbsKafkaProducer.sendDataFetcherJob(cpr, jobType, jobId, msgId);

        verify(mockNspProducer).send(producerRecordArgumentCaptor.capture());

        ProducerRecord<String, DataFetcherJob> producerRecord = producerRecordArgumentCaptor.getValue();
        assertEquals(topic, producerRecord.topic());
        assertEquals(cpr, producerRecord.key());
        assertEquals(cpr, producerRecord.value().getCpr());
        assertEquals(jobType, producerRecord.value().getType());
        assertEquals(jobId, producerRecord.value().getJobId());
        assertEquals(msgId, producerRecord.value().getMsgId());
    }

    @Test
    void test_shutdown() {
        CbsKafkaProducer cbsKafkaProducer = new CbsKafkaProducer("topic", new Properties(), mockFactory);
        assertDoesNotThrow(cbsKafkaProducer::shutdown);
    }
}
