/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka.serialization;

import dk.sundhedsdatastyrelsen.cbs.kafka.model.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.kafka.model.DataFetcherJobBuilder;
import dk.sundhedsdatastyrelsen.cbs.kafka.model.JobType;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import dk.sundhedsdatastyrelsen.cbs.test.TestUtils;
import org.junit.jupiter.api.Test;

import java.util.UUID;

import static com.google.common.truth.Truth.assertThat;

@CbsTest
class JacksonSerializerTest {

    @Test
    void test_serialization() {

        JacksonSerializer<DataFetcherJob> serializer = new JacksonSerializer<>();

        String cpr = TestUtils.randomCprNumber();
        String jobId = UUID.randomUUID().toString();
        String messageId = "MyMessageId";
        JobType jobType = JobType.FETCH_ALL;
        DataFetcherJob job = DataFetcherJobBuilder.aDataFetcherJob()
                .withCpr(cpr)
                .withJobId(jobId)
                .withMsgId(messageId)
                .withType(jobType)
                .build();

        byte[] bytes = serializer.serialize("unused", job);
        String json = new String(bytes);
        assertThat(json).contains(cpr);
        assertThat(json).contains(jobId);
        assertThat(json).contains(messageId);
        assertThat(json).contains(jobType.toString());
    }
}
