/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc;

import dk.sundhedsdatastyrelsen.cbs.dao.Record;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.time.LocalDateTime;
import java.util.UUID;

class RecordImpl<T, U extends RecordImpl<T, U>> implements Record<T> {

    static final String KEY_COLUMN_LABEL = "KEY";
    static final String CREATION_TIME_COLUMN_LABEL = "CREATION_TIME";
    static final String TRACE_ID_COLUMN_LABEL = "TRACE_ID";
    static final String INVALIDATION_TIME_COLUMN_LABEL = "INVALIDATION_TIME";
    static final String DATA_COLUMN_LABEL = "DATA";

    @NonNull private final U actual;
    private UUID key;
    private LocalDateTime creationTime;
    private String traceId;
    private LocalDateTime invalidationTime;
    private T data;

    @SuppressWarnings("unchecked")
    RecordImpl() {
        actual = (U) this;
    }

    @Override
    public UUID key() {
        return key;
    }

    @NonNull
    public U key(@NonNull final UUID key) {
        this.key = key;
        return actual;
    }

    @Override
    public String traceId() {
        return traceId;
    }

    @NonNull
    public U traceId(@NonNull final String traceId) {
        this.traceId = traceId;
        return actual;
    }

    @Override
    public LocalDateTime creationTime() {
        return creationTime;
    }

    @NonNull
    public U creationTime(@NonNull final LocalDateTime creationTime) {
        this.creationTime = creationTime;
        return actual;
    }

    @Override
    public LocalDateTime invalidationTime() {
        return invalidationTime;
    }

    @NonNull
    public U invalidationTime(@Nullable final LocalDateTime invalidationTime) {
        this.invalidationTime = invalidationTime;
        return actual;
    }

    @Override
    public T data() {
        return data;
    }

    @NonNull
    public U data(@Nullable final T data) {
        this.data = data;
        return actual;
    }
}
