/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datafetcher.ddv;

import dk.sundhedsdatastyrelsen.cbs.ddv.*;
import dk.vaccinationsregister.schemas._2021._04._01.*;
import dk.vaccinationsregister.schemas._2021._04._01.SubscriptionType;

import javax.xml.datatype.XMLGregorianCalendar;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

public class DDV2DTOMapper {

    public Covid19VaccinationCard mapEffectuatedItems(Covid19VaccinationCardType source) {
        if (source == null) return null;
        Covid19VaccinationCard target = new Covid19VaccinationCard();
        target.setPersonCivilRegistrationIdentifier(source.getPersonCivilRegistrationIdentifier());
        target.setEffectuatedItem(mapEffectuatedItems(source.getEffectuatedItem()));
        target.setSubscription(mapSubscriptions(source.getSubscription()));
        target.setVaccinationPlan(mapVaccinations(source.getVaccinationPlan()));
        return target;
    }

    private List<VaccinationPlan> mapVaccinations(List<VaccinationPlanType> source) {
        return source.stream().map(this::map).collect(Collectors.toList());
    }

    private VaccinationPlan map(VaccinationPlanType source) {
        if (source == null) return null;
        VaccinationPlan target = new VaccinationPlan();
        target.setPlannedVaccination(mapPlannedVaccinations(source.getPlannedVaccination()));
        target.setVaccinationPlanCode(source.getVaccinationPlanCode());
        target.setVaccinationPlanIdentifier(source.getVaccinationPlanIdentifier());
        target.setVaccinationPlanName(source.getVaccinationPlanName());
        return target;
    }

    private List<PlannedVaccination> mapPlannedVaccinations(List<PlannedVaccinationType> source) {
        return source.stream().map(this::map).collect(Collectors.toList());
    }

    private PlannedVaccination map(PlannedVaccinationType source) {
        if (source == null) return null;
        PlannedVaccination target = new PlannedVaccination();
        target.setEffectuatedItem(map(source.getEffectuatedItem()));
        target.setPlannedVaccinationDateTime(map(source.getPlannedVaccinationDateTime()));
        target.setPlannedVaccinationIdentifier(source.getPlannedVaccinationIdentifier());
        target.setPlannedVaccinationIndex(source.getPlannedVaccinationIndex());
        target.setPlannedVaccinationVersionIdentifier(source.getPlannedVaccinationVersionIdentifier());
        target.setVaccineIdentifier(source.getVaccineIdentifier());
        target.setVaccineName(source.getVaccineName());
        return target;
    }

    private LocalDateTime map(XMLGregorianCalendar source) {
        if (source == null) return null;
        return source.toGregorianCalendar().toZonedDateTime().toLocalDateTime();
    }

    private EffectuatedItem map(EffectuatedItemType source) {
        if (source == null) return null;
        EffectuatedItem target = new EffectuatedItem();
        target.setEffectuatedVaccinationDateTime(map(source.getEffectuatedVaccinationDateTime()));
        target.setEffectuatedVaccinationIdentifier(source.getEffectuatedVaccinationIdentifier());
        target.setEffectuatedVaccinationVersionIdentifier(source.getEffectuatedVaccinationVersionIdentifier());
        target.setEffectuatedVaccineIdentifier(source.getEffectuatedVaccineIdentifier());
        return target;
    }

    private List<Subscription> mapSubscriptions(List<SubscriptionType> subscription) {
        return null;
    }

    private List<EffectuatedItem> mapEffectuatedItems(List<EffectuatedItemType> effectuatedItem) {
        return null;
    }

}
