/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.webservice;

import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.*;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

@Service
public class MockDataService {

    private static Map<String, String> jwts = new HashMap<>();

    static {
        jwts.put("DK", "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJleHAiOjE2MTc3MTg3MTYsImlhdCI6MTYxNjQwMzY1NywianRpIjoiOWQwMjhmMjktMjU0Ni00ZDU5LThjN2EtOGJjOWI1MmEzNmU4In0.c-FZaQWVoosnJA18uoOItgX10v9XHDHkAyNgfDgdipQ");
        jwts.put("DKPLUS", "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJleHAiOjE2MTc3MTg4MTEsImlhdCI6MTYxNjQwMzY1NywianRpIjoiOWQwMjhmMjktMjU0Ni00ZDU5LThjN2EtOGJjOWI1MmEzNmU4IiwibGVnYWxOYW1lIjoiSmFudXMgTmllbHNlbiIsImRhdGVPZkJpcnRoIjoiMTk4NS0xMi0wMyJ9.YQpCr3igO61wBjcrSIsIXGp_5FBTV0KQUTw-ZrZDEOg");
        jwts.put("EU", "6BFC$B5Z7+J2M433FH*:R71FZ7F+IMVC3L6Q6WCPCGU+CLGE86U9.PK0E5QV2UF$XCL66N93/.DMNK4FO/34*LGTP2K+Q/RO602W5P%H9H%RM5A8I1$CAROQC1252AQIS*ZUB17+K74VMWZS8/E8V2CY2M2HO8J:0H1XI$E5KCOCC07NB9EB$KC0:2W-R$*2RYRQ6PW.VYKVUNC4QK4*U7+QZ3KFTOK56LSLQV4JNB0ML7RINLGBPK7U7T.4PNI%CBEY3ECF7W37XK.73JA9IQGKOKK62 9VPG0*CIFGBLK5P*I$DNCE4PN8LFNO769AHBILWRSPA644AYKUOOL$MRIFCUR0+.2%9J$/2VHS1ML325 80E:2Z:C1 TI8PP$P84E3-3CNDD2C/4ED D.3NL9BW7P.BQQZFE8QI 1Q9G/ECHW8NS4VN6.%7JMN:Y8JLUY9DEF90319JF4A8.TS+4NSDMHD1:-JW 02DA5*P4$OWMAHLCJ2Q266JA0TNBC0QWOBO$TO8QM%A5P7QVPN3HUBAHJF0KA EUPH2N-JEM753B4KRS3CU.7FYNBUCVND*9E7VJS:VZBQH2GWTDV2JQHGZ7O6:36F6+$RU/UEUR76IY17XZN706YCFS FMCSO04R38C4OA$VI0C470/J9H1");
    }

    public GetCoronaPassResponse cacheMiss(String jobId) {
        GetCoronaPassResponse getCoronaPassResponse = new GetCoronaPassResponse();
        FutureResult futureResult = new FutureResult();
        futureResult.setJobId(jobId);
        getCoronaPassResponse.setFutureResult(futureResult);

        return getCoronaPassResponse;
    }


    public GetCoronaPassResponse cacheHit() {
        GetCoronaPassResponse getCoronaPassResponse = new GetCoronaPassResponse();
        getCoronaPassResponse.setCoronaPasses(new PassList());
        for (String type : jwts.keySet()) {
            PassJWT cp1 = new PassJWT();
            cp1.setType(type);
            cp1.setJWT(jwts.get(type));
            if ("EU".equals(type)) {
                KeyValuePair kv = new KeyValuePair();
                kv.setKey("eu:keytype");
                kv.setValue("digital_green_certificate");
                cp1.getProperties().add(kv);
            }
            getCoronaPassResponse.getCoronaPasses().getPass().add(cp1);
        }
        return getCoronaPassResponse;
    }
}
